#!/usr/bin/env python
# -*- coding: UTF-8 -*-

# Script para descargar archivos desde goear.
# Versión: 0.5
# Copyright (C) 2007-2010 Daniel Fuentes B.

# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

# Importamos algunos módulos que luego usaremos en el script:

import sys
import urllib
import urllib2
from xml.dom import minidom

# El texto que se muestra en la ayuda
texto_ayuda = """Script para descargar archivos de goear
Modo de uso:
goear-dl.py [Opción] [Audio_Url o ID] 

Donde [Audio_Url] es la dirección (o url) del archivo, la cual es
de la siguiente forma:

http://www.goear.com/listen.php?v=xxxxxxx
o
http://www.goear.com/listen/xxxxx/cancion-artista

También se puede ingresar la ID directamente en vez de la url (la
ID es un código de 7 números/letras intercalados).

Y en [Opción] se puede ingresar una de las siguientes opciones:

Opciones disponibles:
-h, --help        Muestra la ayuda
-v, --version     Muestra la versión del programa
-q, --quiet       Omite el progreso de la descarga

Ejemplos:

goear-dl.py http://www.goear.com/listen.php?v=xxxxxxx
goear-dl.py http://www.goear.com/listen/xxxxx/cancion-artista
 # Descarga el archivo desde esa url 

goear-dl.py -q http://www.goear.com/listen.php?v=xxxxx
goear-dl.py -q http://www.goear.com/listen/xxxxx/cancion-artista
 # Descarga sin mostrar el progreso

goear-dl.py xxxxxxx    # Descarga ingresando solo a ID (que es xxxxxxx)

Para mayor información revise el archivo "README" """

# ============================================================================
# Funciones y clases usadas en el programa:
# ============================================================================

# Definimos la función de descarga:
def download_file(filename, URL, progreso=True):
    if progreso == False:
        f = urllib.urlretrieve(URL, filename)
    else:
        f = urllib.urlretrieve(URL, filename, reporthook=my_report_hook)

# Definimos la función que muestra el progreso del archivo:
# Usada si al descargar se pide mostrar el progreso
def my_report_hook (block_count, block_size, total_size): 
    total_kb  = total_size / 1024
    descargado = block_count * (block_size/1024)
    print "Descargados %d kb de %d kb" %(descargado,total_kb)

# ----------------------------------------------------------------------------
# Clase que muestra la informacion del archivo
class Goear():
    "Clase que muestra la informacion del archivo (nombre, artista, etc)"
    def __init__(self, audio_id):
        self.id = audio_id
        self.data = None
        # Para que funcione la clase, al incio hay que obtener la informacion
        # del archivo xml a partir de la ID (primero abrir el xml)
        url_xml = "http://www.goear.com/localtrackhost.php?f=" + self.id
        # Nota: Normalmente cambian la linea anterior para que los
        # programas no funcionen al no encortrar el xml
        try:
            xmldoc = urllib2.urlopen(url_xml, None)
            xml_data = minidom.parse(xmldoc)
            xmldoc.close()
            data = xml_data.toxml()
        except urllib2.HTTPError: # Mensaje en caso de error
            print "Error, no se encuentra la información necesaria\n"
            print "Esto puede ser provocado por ingresar mal la dirección (Url) del archivo,"
            print "por ingresar mal la ID o por ingresar una opción no valida."
            print "Por favor revise la ayuda con la opción --help"
            sys.exit()
        self.data = data # Contiene la copia de informacion del xml

    def get_id(self):
        "muestra la ID de la cancion"
        return self.id

    def url_file(self):
        "devuelve la url del archivo mp3"
        # Buscar la linea "song path=" que es la dirección del archivo
        inicio_path = self.data.find("path=")
        fin_path = self.data.find(".mp3")
        path = self.data[inicio_path+6:fin_path+4]
        return path

    def artist(self):
        "devuelve el grupo/artista de esta cancion"
        # Buscar la linea "title=" que es la dirección del archivo
        inicio = self.data.find("artist=")
        fin = self.data.find("\"",(inicio+8),-1)
        artist = self.data[inicio+8:fin]
        return artist

    def title(self):
        "devuelve el titulo de la cancion"
        # Buscar la linea "title=" que es la dirección del archivo
        inicio = self.data.find("title=")
        fin = self.data.find("\"",(inicio+7),-1)
        title = self.data[inicio+7:fin]
        return title

# ----------------------------------------------------------------------------
# Funcion que obtiene la ID a partir de la dirección ingresada

def get_id(url):
    # hay varios posibles casos:
    # a) Se ingresa directamente la ID (codigo de 7 numeros/letras)
    # b) De la forma antigua (http://www.goear.com/listen.php?v=xxxxx):
    # c) De la forma nueva (http://www.goear.com/listen/xxxxx/cancion-artista)

    if len(audio_url) == 7:
        # se ingreso la id directamente (primer caso)
        audio_id = audio_url
        return audio_id

    elif (audio_url.find("listen.php?v=")) != -1: # Segundo caso
        # de la forma antigua el find deberia encontrar el "listen.php?v="
        # siendo distintinto de -1 (es -1 cuando no encuentra el texto pedido)
        inicio_id = audio_url.find("v=")
        audio_id = audio_url[inicio_id+2:+inicio_id+9]
        return audio_id

    else:
        # la nueva forma (tercer caso)
        inicio_id = audio_url.find("listen/")
        audio_id = audio_url[inicio_id+7:inicio_id+14]
        return audio_id


#=============================================================================
# Fin de las funciones/ clases
#=============================================================================

# Algunas variables a usar:

progreso = True
opciones_ingresadas = []

# Comprobar a continuación si se ingreso algun dato en la linea de ordenes
# Si no se ingresa nada sys.argv solo tendra el nombre del script (goear-dl.py)

if len(sys.argv) == 1: # El unico dato es goear-dl.py (no hay ordenes o url)
    tengo_opciones = False
    audio_url = str(raw_input("Ingrese la dirección (url) del archivo o su ID: "))

elif len(sys.argv) == 2: 
# Dos casos posibles: se ingreso una opcion o la url (dirección)
# Además si la opcion es "-q" (caso 1) hay que preguntar la url 
    if sys.argv[1] == "-q" or sys.argv[1] == "--quiet":
        tengo_opciones = True
        opciones_ingresadas.append(sys.argv[1])
        audio_url = str(raw_input("Ingrese la dirección (url) del archivo o su ID: "))
    elif sys.argv[1] == "-h" or sys.argv[1] == "--help" or sys.argv[1] == "help" or sys.argv[1] == "-v" or sys.argv[1] == "--version":
        tengo_opciones = True
        opciones_ingresadas.append(sys.argv[1])
        audio_url = None
    elif sys.argv[1] == "moo":
        print "No hay ninguna Súper Vaca en este programa, pero"
        print "tenemos una danza"
        print "<(^.^<)   <(^-^)>   (>^.^)>"
        sys.exit()
    else:
        tengo_opciones = False
        audio_url = sys.argv[1]

elif len(sys.argv) > 2:
# Casos posibles: tener solo ordenes o tener opciones (ordenes) + dirección
    tengo_opciones = True
    for n in sys.argv[1:len(sys.argv)]:
        if n == "-q" or n == "--quiet" or n == "-h" or n== "--help" or n == "help" or n== "-v" or n== "--version":
            opciones_ingresadas.append(n)
            audio_url = None
        else:
            audio_url = n

# Verificar Opciones de la linea de comandos
# Y proceder de acuerdo a ellas
if tengo_opciones:
    for opcion in opciones_ingresadas:
        if opcion == "-h" or opcion == "--help" or opcion == "help":
            print texto_ayuda                    
            sys.exit()
        elif opcion == "-v" or opcion == "--version":
            print "Versión 0.5"
            sys.exit()
        elif opcion == "-q" or opcion == "--quiet":
            progreso = False

# Comprobar la dirección (url) del audio sea correcta:
# Puede darse casos como el que sigue: goear.py -q -q
# En este caso aun no se ingresa la direción 
# (pero el usuario no debería ingresar eso, ya que no es correcto)
if audio_url == None:
    audio_url = str(raw_input("Ingrese la dirección (url) del archivo o su ID: "))


# Ahora hay que juntar todo:
if __name__ == "__main__":
    audio_id = get_id(audio_url)
    print "Iniciando descarga de:", audio_id
    cancion = Goear(audio_id)
    # Guargdamos el nombre del archivo
    nombre_archivo = cancion.get_id() + " - " + cancion.title() + " - "+ cancion.artist() + ".mp3"
    download_file(nombre_archivo,cancion.url_file(), progreso)
    print "Listo, ahora solo abra el archivo llamado:\n\"%s\"" % (nombre_archivo)

